"use strict";

const ee = require("./eventemitter");

const body = window.document.body;
let layers = [];

/**
 * Register a layer
 *   el: An object with an "isOpen" method (required)
 *   closedEventName: The name of the event that's emitted when the layer has finished closing (required)
 */
function registerLayer(el, closedEventName) {
  layers.push({
    "element": el,
    "closedEventName": closedEventName
  });
  ee.addListener(closedEventName, onLayerClosed);
}

function unregisterLayer(el) {
  let filtered = layers.filter((layer, index, array) => {
    if (layer.element === el) {
      ee.removeListener(layer.closedEventName, onLayerClosed);
      return false;
    }
    return true;
  });
}

/**
 * When a layer has finished closing, check whether it's safe to allow
 * the body to scroll again (all registered layers must be closed)
 */
function onLayerClosed() {
  let allLayersClosed = true;
  layers.map((currentValue, index, array) => {
    if (currentValue.element.isOpen()) {
      allLayersClosed = false;
    }
  });
  if (allLayersClosed) {
    allowScrolling();
  }
}

function preventScrolling() {
  body.classList.add("u-no-scroll");
}

function allowScrolling() {
  body.classList.remove("u-no-scroll");
}

module.exports = {
  registerLayer,
  unregisterLayer
};
